version 11.2
clear all
set more off
set mem 50m
capture log close

* Change the file path to the directory where the replication package folder is
cd ".../FK_2016_replication"

log using Log/Figure_1_simulation.log, replace text

***********************************************************************
* 
* 'Testing under a special form of heteroscedasticity', Farbmacher and Koegel
* 
* This do file replicates the Monte Carlo Simulation results and reproduces
* the illustration of the simulation results in Figure 1
*
***********************************************************************

***
* 1) Define the program generating the sample for each repetition of the simulation
***

program define dgp, rclass
	syntax, h(real) obs(real) xmean(real) xstd(real) numerator(real) exponent(real) ybetax(real) constant(real)
	qui {
		drop _all
		scalar drop _all

		*** 1A) Generate the dataset

		* Set number of observations
		set obs `obs'

		* Generate the x variable
		gen x=rnormal(`xmean',`xstd')

		* Transform the x variable into its rounded version (integers)
		replace x=round(x)

		* Generate the squared x variable
		gen x2=x^2

		* Generate the squared demeaned x variable
		sum x
		gen xmod=x-r(mean)
		gen xmod2=xmod^2
		drop xmod

		* Generate epsilon in the error term e
		gen epsilon=rnormal(0,1)

		* Generate the error term e
		gen e=sqrt((`numerator'/((xmod2+`h')^`exponent')))*epsilon

		* Generate the y variable
		gen y=`constant'+`ybetax'*x+e

		* Predict the (squared) residuals from the regression of y on x
		if `constant'!=0 reg y x
		if `constant'==0 reg y x, nocon
		predict resid, resid
		gen resid2=resid^2

		*** 1B) Obtain the parameters of interest

		* Obtain the conventional standard error of beta and the beta point estimate
		if `constant'!=0 reg y x
		if `constant'==0 reg y x, nocon
		matrix eV=e(V)
		scalar sdbeta=sqrt(eV[1,1])
		matrix eb=e(b)
		scalar beta=eb[1,1]

		* Obtain the robust standard error of beta
		if `constant'!=0 reg y x, robust
		if `constant'==0 reg y x, robust nocon
		matrix eV=e(V)
		scalar sdbeta_robust=sqrt(eV[1,1])

		*** 1C) Perform the heteroscedasticity tests

		* Koenker's version of the Breusch-Pagan test and the White test
		
		/*
			Note:
			- Koenker's version of the Breusch-Pagan (BP) test is algebraically equivalent to the White test when using
			  the same regressors in Koenker's version of the BP test as the ones used in the White test (Waldman, D. M., 1983.
			  "A note on algebraic equivalance of White's test and a variation of the Godfrey/Breusch-Pagan test for
			  heteroskedasticity", Economics Letters, 13(2-3), pp. 197-200).
			  For this reason, we can use the commands hettest with the iid option not only to obtain Koenker's
			  version of the BP test but also the White test.
		*/

		reg y x

		hettest, rhs iid
		scalar hettest_pvalue_rhs_iid=r(p)
		
		hettest x x2, iid
		scalar hettest_pvalue_x_x2_iid=r(p)

		* Our Elliptical Heteroscedasticity test
		reg resid2 xmod2, robust
		scalar t_robust_xmod2=_b[xmod2]/_se[xmod2]
		scalar beta_xmod2_robust=_b[xmod2]
	}
end

***
* 2) Run the simulation
***

* Perform the simulation with N=214 and N=2140
foreach N of numlist 214 2140 {

	* Define the parameters for the data generating process
	local xmean 0.04
	local xstd 1.8
	local ybetax 0.04
	local numerator 1
	local constant 0

	* Set range and steps of a values
	local a_min 0
	local a_max 0.5
	local a_steps 0.01

	* Number of repetitions in the simulation
	local reps 10000
	local obs `N'

	* Define matrices in which all simulation results of interest are displayed at the end of this do file
	matrix define power`N'=(0,0,0)

	matrix define a=(0)
	matrix define ps`N'=(0)
	matrix define ps_rob`N'=(0)

	foreach num0 of numlist `a_min'(`a_steps') `a_max' {
		local exponent `num0'

		#delimit
		simulate
		
		beta=beta
		sdbeta=sdbeta
		sdbeta_robust=sdbeta_robust
		hettest_pvalue_rhs_iid=hettest_pvalue_rhs_iid
		hettest_pvalue_x_x2_iid=hettest_pvalue_x_x2_iid
		t_robust_xmod2=t_robust_xmod2,
		
		seed(010101)
		reps(`reps'):

		dgp, h(0.1) obs(`obs') xmean(`xmean') xstd(`xstd') numerator(`numerator') exponent(`exponent') ybetax(`ybetax') constant(`constant');

		#delimit cr

		* Calculate the t-values for the hypotheses tests of beta=0.04 in the regression of y on x, using conventional and robust standard errors
		gen t_b_4=(beta-0.04)/sdbeta
		gen t_b_robust_4=(beta-0.04)/sdbeta_robust
	
		* Generate dummy variables indicating whether H0: beta=0.04 is rejected at the 5% level based on the t-values calculated immediately above
		foreach num of numlist 0 {
			gen rej_t_b_4=1 if abs(t_b_4)>invnormal(0.975)
			replace rej_t_b_4=0 if abs(t_b_4)<=invnormal(0.975)
			gen rej_t_b_robust_4=1 if abs(t_b_robust_4)>invnormal(0.975)
			replace rej_t_b_robust_4=0 if abs(t_b_robust_4)<=invnormal(0.975)
		}
		
		* Compute the rejection rate for each hypothesis test of beta=0 and put it in a scalar variable
		foreach var2 of varlist rej_t_b_4 rej_t_b_robust_4 {
			qui sum `var2'
			scalar m_`var2'=r(mean)
		}

		* Generate dummy variables indicating whether the heteroscedasticity tests reject H0
		foreach var of varlist hettest_pvalue_rhs_iid hettest_pvalue_x_x2_iid {
			gen reject_`var'=1 if `var'<0.05
			replace reject_`var'=0 if `var'>=0.05
		}
		gen reject_t_robust_xmod2=1 if t_robust_xmod2<invnormal(0.05)
		replace reject_t_robust_xmod2=0 if t_robust_xmod2>=invnormal(0.05)

		* Compute the rejection rate for each heteroscedasticity test and put it in a scalar variable
		foreach var2 of varlist reject_hettest_pvalue_rhs_iid reject_hettest_pvalue_x_x2_iid reject_t_robust_xmod2 {
			qui sum `var2'
			scalar m_`var2'=r(mean)
		}

		* Put the rejection rates in the corresponding matrices
		matrix define a=(a\ `num0')
		matrix define power`N'=(power`N'\m_reject_hettest_pvalue_rhs_iid, m_reject_hettest_pvalue_x_x2_iid, m_reject_t_robust_xmod2)
		matrix define ps`N'=(ps`N'\m_rej_t_b_4)
		matrix define ps_rob`N'=(ps_rob`N'\m_rej_t_b_robust_4)
	}
}

***
* 3) Display in matrices the simulation results illustrated in Figure 1
***

foreach N of numlist 214 2140 {
	matrix define power`N'=(a,power`N')
	scalar rowspower`N'=rowsof(power`N')
	scalar colspower`N'=colsof(power`N')
	matrix define power`N'=(power`N'[2..rowspower`N',1..colspower`N'])
	matrix colnames power`N'= a KBP White "Ell Het"

	matrix define size`N'=(a, ps`N', ps_rob`N')
	scalar rows`N'=rowsof(size`N')
	scalar cols`N'=colsof(size`N')
	matrix define size`N'=(size`N'[2..rows`N',1..cols`N'])
	matrix colnames size`N'= a "conv SE" "robust SE"
}

*** 3A) Figure 1, Panel A: Power plots for heteroscedasticity tests

* N=214
matrix list power214

* N=2140
matrix list power2140

*** 3B) Figure 1, Panel B: Actual size for Wald tests using robust and conventional SEs

* N=214
matrix list size214

* N=2140
matrix list size2140

***
* 4) Reproduce the graphs in Figure 1 (and add legends and headings)
***

clear

* Create variables from the matrices defined above
qui svmat power214
qui svmat power2140
qui svmat size214
qui svmat size2140

* Variable for creating the shaded area in the graphs
gen peak1=1
gen peak2=0.06

*** 4A) Figure 1, Panel A

* N=214
graph twoway (area peak1 power2141 if power2141>=.15 & power2141 <=.301, lcolor(gs13) fcolor(gs13) ) ///
(line power2142 power2141 if power2141 <52, lpattern(solid)) (line power2143 power2141 if power2141 <52, lpattern(----)) ///
 (line power2144 power2141 if power2141 <52) (function y=0.05, range(0 0.5) lpattern(dot)), plotregion(margin(l=.5 b=.5)) ///
legend(order(2 "Breusch-Pagan Test" 3 "White Test" 4 "Elliptical Het. Test")) ymlabel(0.05, angle(0)) xmlabel(0.15) xtitle(a) ///
scheme(s2mono) ylabel(0(0.1)1, angle(0)) ytitle(Rejection Frequency) title("Figure 1, Panel A, N=214") name(Fig1PanA214, replace)

* N=2140
graph twoway (area peak1 power21401 if power21401>=.15 & power21401 <=.301, lcolor(gs13) fcolor(gs13) ) ///
(line power21402 power21401 if power21401 <52, lpattern(solid)) (line power21403 power21401 if power21401 <52, lpattern(----)) ///
 (line power21404 power21401 if power21401 <52) (function y=0.05, range(0 0.5) lpattern(dot)), plotregion(margin(l=.5 b=.5)) ///
legend(order(2 "Breusch-Pagan Test" 3 "White Test" 4 "Elliptical Het. Test")) ymlabel(0.05, angle(0)) xmlabel(0.15) xtitle(a) ///
scheme(s2mono) ylabel(0(0.1)1, angle(0)) ytitle(Rejection Frequency) title("Figure 1, Panel A, N=2140") name(Fig1PanA2140, replace)

*** 4B) Figure 1, Panel B

* N=214
graph twoway (area peak2 size2141 if size2141>=.15 & size2141 <=.301, lcolor(gs13) fcolor(gs13) ) ///
(line size2142 size2141 if size2141 <52, lpattern(solid)) (line size2143 size2141 if size2141 <52, lpattern(dash_dot)) ///
(function y=0.05, range(0 0.5) lpattern(----)), plotregion(margin(l=.5 b=.5)) legend(order(2 "conventional SE" 3 "robust SE")) ///
xmlabel(0.15) xtitle(a) scheme(s2mono) ylabel(0(0.01)0.06, angle(0)) ytitle(Actual Size) title("Figure 1, Panel B, N=214") name(Fig1PanB214, replace)

* N=2140
graph twoway (area peak2 size21401 if size21401>=.15 & size21401 <=.301, lcolor(gs13) fcolor(gs13) ) ///
(line size21402 size21401 if size21401 <52, lpattern(solid)) (line size21403 size21401 if size21401 <52, lpattern(dash_dot)) ///
(function y=0.05, range(0 0.5) lpattern(----)), plotregion(margin(l=.5 b=.5)) legend(order(2 "conventional SE" 3 "robust SE")) ///
xmlabel(0.15) xtitle(a) scheme(s2mono) ylabel(0(0.01)0.06, angle(0)) ytitle(Actual Size) title("Figure 1, Panel B, N=2140") name(Fig1PanB2140, replace)

log close
