version 11.2
clear all
set more off
set mem 50m
capture log close

* Change the file path to the directory where the replication package folder is
cd ".../FK_2016_replication"

log using Log/Table_1_hettests.log, replace text

***********************************************************************
* 
* 'Testing under a special form of heteroscedasticity', Farbmacher and Koegel
* 
* This do file replicates the heteroscecedasticity tests in Table 1
*
***********************************************************************


***
* 1) Prepare data set based on the original data set used by Bonjour et al. (2003) and Amin (2011)
***
use Data/BCHHS_data.dta, clear

* Generate log earnings
gen lnearn=ln(earning)

* Generate age squared
gen agesq=age*age

* Generate within-twin differences
reshape wide earning-agesq, i(family) j(twinno)
gen dlnearn = lnearn1-lnearn2
gen dhigh = highqua1-highqua2
gen dtwihi=twihigh1-twihigh2
gen dwage=earning1-earning2
la var dwage "within-twin difference in wages"
gen awage=abs(dwage)
la var awage "absolute within-twin differences in wages"
gen dpart = part1-part2
gen dmarried = married1-married2
gen dLNandSE = LNandSE1-LNandSE2
gen down_exp = own_exp1-own_exp2

* Generate indicator for observations to use in the regressions including covariates
gen include= 1-(dlnearn==.|dhigh==.|dtwihi==.|dpart==.|dmarried==.|dln==.|down_exp==.)

/*  Note:
	- The restriction awage<=999 does not exclude any observations from the estimations
	  and yields thus the full sample estimates
	- The restrictions awage<=90, 75, 65, 60 exclude all observations with an absolute within-twin
	  wage difference greater than 90, 75, 65, 60 pounds
*/

* Generate the relevant residuals for the tests below
* For tests in column 3
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dlnearn dhigh if awage<=`x', noconstant
		predict resid_3_d_ols_nc_`x' if e(sample)==1, resid
	}
}
* For tests in column 4
foreach x of numlist 999 90 75 65 60 {
	qui {
		ivreg dlnearn (dhigh=dtwihi) if awage<=`x', noconstant
		predict resid_4_d_iv_nc_`x' if e(sample)==1, resid
	}
}
* For tests in column 7
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dlnearn dhigh dLNandSE dpart dmarried down_exp if awage<=`x' & include==1, noconstant
		predict resid_7_d_ols_c_`x' if e(sample)==1 & include==1, resid
	}
}
* For tests in column 8
foreach x of numlist 999 90 75 65 60 {
	qui {
		ivreg dlnearn (dhigh=dtwihi) dLNandSE dpart dmarried down_exp if awage<=`x' & include==1, noconstant
		predict resid_8_d_iv_c_`x' if e(sample)==1 & include==1, resid
	}
}

* Partial out the relevant variables for the tests below
* dhigh
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dhigh dLNandSE dpart dmarried down_exp if awage<=`x' & include==1, noconstant
		predict partialled_d_dhigh_`x' if e(sample)==1 & include==1, resid
	}
}
* dlnearn
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dlnearn dLNandSE dpart dmarried down_exp if awage<=`x' & include==1, noconstant
		predict partialled_d_dlnearn_`x' if e(sample)==1 & include==1, resid
	}
}
* dtwihi
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dtwihi dLNandSE dpart dmarried down_exp if awage<=`x' & include==1, noconstant
		predict partialled_d_dtwihi_`x' if e(sample)==1 & include==1, resid
	}
}

* Generate squared residuals
foreach x of varlist resid_3_d_ols_nc_999 - resid_8_d_iv_c_60 {
	qui gen sq_`x'=`x'*`x'
}

* Generate additional squared variables and squared demeaned variables
foreach var of varlist dhigh partialled_d_dhigh_999-partialled_d_dhigh_60 {
	qui gen `var'2=`var'^2
	qui sum `var'
	qui gen `var'mod2=(`var'-r(mean))^2
}

***
* 2) Perform Koenker's version of the Breusch-Pagan test and the White test (and put the p values in matrices)
***

/*
	Note:
	- Koenker's version of the Breusch-Pagan (BP) test is algebraically equivalent to the White test when using
	  the same regressors in Koenker's version of the BP test as the ones used in the White test (Waldman, D. M., 1983.
	  "A note on algebraic equivalance of White's test and a variation of the Godfrey/Breusch-Pagan test for
	  heteroskedasticity", Economics Letters, 13(2-3), pp. 197-200).
	  For this reason, we can use the commands hettest with the iid option and ivhettest below not only to obtain Koenker's
	  version of the BP test but also the White test.
*/

* Column 3
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg dlnearn dhigh if awage<=`x'
		hettest, rhs iid
		scalar het_3_p_dhigh_iid_`x'=r(p)
		hettest dhigh dhigh2, iid
		scalar het_3_p_dhigh_dhigh2_iid_`x'=r(p)
	}
}

* Column 4
foreach x of numlist 999 90 75 65 60 {
	qui {
		ivreg dlnearn (dhigh=dtwihi) if awage<=`x'
		ivhettest dhigh
		scalar het_4_p_iv_dhigh_`x'=r(php)
		ivhettest dhigh dhigh2
		scalar het_4_p_iv_dhigh_dhigh2_`x'=r(php)
	}
}

* Column 7
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg partialled_d_dlnearn_`x' partialled_d_dhigh_`x' if awage<=`x' & include==1
		hettest, rhs iid
		scalar het_7_p_c_dhigh_iid_`x'=r(p)
		hettest partialled_d_dhigh_`x' partialled_d_dhigh_`x'2, iid
		scalar het_7_p_c_dhigh_dhigh2_iid_`x'=r(p)
	}
}

* Column 8
foreach x of numlist 999 90 75 65 60 {
	qui {
		ivreg partialled_d_dlnearn_`x' (partialled_d_dhigh_`x'=partialled_d_dtwihi_`x') if awage<=`x' & include==1
		ivhettest partialled_d_dhigh_`x'
		scalar het_8_p_c_iv_dhigh_iid_`x'=r(php)
		ivhettest partialled_d_dhigh_`x' partialled_d_dhigh_`x'2
		scalar het_8_p_c_iv_dhigh_dh2_iid_`x'=r(php)
	}
}

* Create a matrix containing the test results for both tests
foreach num of numlist 3 4 7 8 {
	matrix input het_dhigh_`num'=()
	matrix input het_dhigh_dhigh2_`num'=()
}

foreach num of numlist 999 90 75 65 60 {
	matrix define het_dhigh_3=(het_dhigh_3\het_3_p_dhigh_iid_`num')
	matrix define het_dhigh_4=(het_dhigh_4\het_4_p_iv_dhigh_`num')
	matrix define het_dhigh_7=(het_dhigh_7\het_7_p_c_dhigh_iid_`num')
	matrix define het_dhigh_8=(het_dhigh_8\het_8_p_c_iv_dhigh_iid_`num')
	matrix define het_dhigh_dhigh2_3=(het_dhigh_dhigh2_3\het_3_p_dhigh_dhigh2_iid_`num')
	matrix define het_dhigh_dhigh2_4=(het_dhigh_dhigh2_4\het_4_p_iv_dhigh_dhigh2_`num')
	matrix define het_dhigh_dhigh2_7=(het_dhigh_dhigh2_7\het_7_p_c_dhigh_dhigh2_iid_`num')
	matrix define het_dhigh_dhigh2_8=(het_dhigh_dhigh2_8\het_8_p_c_iv_dhigh_dh2_iid_`num')
}
matrix input awage=(999\90\75\65\60)

matrix define KBP=(awage, het_dhigh_3, het_dhigh_4, het_dhigh_7, het_dhigh_8)
matrix colnames KBP="Restriction" "Column 3" "Column 4" "Column 7" "Column 8"

matrix define White=(awage, het_dhigh_dhigh2_3, het_dhigh_dhigh2_4, het_dhigh_dhigh2_7, het_dhigh_dhigh2_8)
matrix colnames White="Restriction" "Column 3" "Column 4" "Column 7" "Column 8"

***
* 3) Perform our Elliptical Heteroscedasticity test (and put the p values in a matrix)
***

* Column 3
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg sq_resid_3_d_ols_nc_`x' dhighmod2, robust
		scalar t_3_rob_ols_dhighmod2_`x'=_b[dhighmod2]/_se[dhighmod2]
	}
}

*Column 4
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg sq_resid_4_d_iv_nc_`x' dhighmod2, robust
		scalar t_4_rob_iv_dhighmod2_`x'=_b[dhighmod2]/_se[dhighmod2]
	}
}

* Column 7
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg sq_resid_7_d_ols_c_`x' partialled_d_dhigh_`x'mod2, robust
		scalar t_7_par_rob_ols_dhighmod2_`x'=_b[partialled_d_dhigh_`x'mod2]/_se[partialled_d_dhigh_`x'mod2]
	}
}

* Column 8
foreach x of numlist 999 90 75 65 60 {
	qui {
		reg sq_resid_8_d_iv_c_`x' partialled_d_dhigh_`x'mod2, robust
		scalar t_8_par_rob_iv_dhighmod2_`x'=_b[partialled_d_dhigh_`x'mod2]/_se[partialled_d_dhigh_`x'mod2]
	}
}

* Create a matrix containing the test results
foreach num of numlist 3 4 7 8 {
	matrix input p_dhighmod2_`num'=()
}

matrix input p_dhighmod2=()

foreach num of numlist 999 90 75 65 60 {
	matrix define p_dhighmod2_3=(p_dhighmod2_3\normal(t_3_rob_ols_dhighmod2_`num'))
	matrix define p_dhighmod2_4=(p_dhighmod2_4\normal(t_4_rob_iv_dhighmod2_`num'))
	matrix define p_dhighmod2_7=(p_dhighmod2_7\normal(t_7_par_rob_ols_dhighmod2_`num'))
	matrix define p_dhighmod2_8=(p_dhighmod2_8\normal(t_8_par_rob_iv_dhighmod2_`num'))
}

matrix define EllHet=(awage, p_dhighmod2_3, p_dhighmod2_4, p_dhighmod2_7, p_dhighmod2_8)
matrix colnames EllHet="Restriction" "Column 3" "Column 4" "Column 7" "Column 8"

***
* 4) Display the test results for all three heteroscedasticity tests
***

* Koenker's version of the Breusch-Pagan test
mat list KBP

* White test
mat list White

* Elliptical Heteroscedasticity test
mat list EllHet

log close
