version 11.2
clear all
set more off
set mem 50m
capture log close

* Change the file path to the directory where the replication package folder is
cd ".../FK_2016_replication"

log using Log/Table_1_returns.log, replace text

use Data/BCHHS_data.dta, replace


***********************************************************************
* 
* 'Testing under a special form of heteroscedasticity', Farbmacher and Koegel
* 
* This do file replicates the return to education estimates, including
* the corresponding conventional and robust standard errors, in Table 1
*
***********************************************************************


***
* 1) Prepare data set
***

* Generate log earnings
gen lnearn=ln(earning)

* Generate age squared
gen agesq=age*age

* Generate within-twin differences
reshape wide earning-agesq, i(family) j(twinno)
gen dlnearn = lnearn1-lnearn2
gen dhigh = highqua1-highqua2
gen dtwihi=twihigh1-twihigh2
gen dwage=earning1-earning2
la var dwage "within-twin difference in wages"
gen awage=abs(dwage)
la var awage "absolute within-twin differences in wages"
gen dpart = part1-part2
gen dmarried = married1-married2
gen dLNandSE = LNandSE1-LNandSE2
gen down_exp = own_exp1-own_exp2

* Generate indicator for observations to use in the regressions including covariates
gen include= 1-(dlnearn==.|dhigh==.|dtwihi==.|dpart==.|dmarried==.|dln==.|down_exp==.)


***
* 2) Replicate the returns to education estimates and their conventional and robust
*	 standard errors in Table 1 (and put the estimates in matrices)
***
/*  Note:
	- The restriction awage<=999 does not exclude any observations from the estimations
	  and yields thus the full sample estimates
	- The restrictions awage<=90, 75, 65, 60 exclude all observations with an absolute within-twin
	  wage difference greater than 90, 75, 65, 60 pounds
*/

	* Column 3
matrix define col3=(0)
foreach num of numlist 999 90 75 65 60 {
	qui {

		di "Column 3, conventional standard errors, awage<=`num'"
		reg dlnearn dhigh if awage<=`num', noconstant
		scalar col3_beta_`num'=_b[dhigh]
		scalar col3_convSE_`num'=_se[dhigh]
	
		di "Column 3, robust standard errors, awage<=`num'"
		reg dlnearn dhigh if awage<=`num', noconstant robust
		scalar col3_robSE_`num'=_se[dhigh]
	
		matrix define col3=(col3\col3_beta_`num'\col3_convSE_`num'\col3_robSE_`num')
	}
}

	* Column 4
matrix define col4=(0)
foreach num of numlist 999 90 75 65 60 {
	qui {
		di "Column 4, conventional standard errors, awage<=`num'"
		ivreg dlnearn (dhigh=dtwihi) if awage<=`num', noconstant `rob`num1''
		scalar col4_beta_`num'=_b[dhigh]
		scalar col4_convSE_`num'=_se[dhigh]
	
		di "Column 4, robust standard errors, awage<=`num'"
		ivreg dlnearn (dhigh=dtwihi) if awage<=`num', noconstant robust
		scalar col4_robSE_`num'=_se[dhigh]
	
		matrix define col4=(col4\col4_beta_`num'\col4_convSE_`num'\col4_robSE_`num')
	}
}

	* Column 7
matrix define col7=(0)
foreach num of numlist 999 90 75 65 60 {
	qui {

		di "Column 7, conventional standard errors, awage<=`num'"
		reg dlnearn dhigh dLNandSE dpart dmarried down_exp if awage<=`num' & include==1, noconstant
		scalar col7_beta_`num'=_b[dhigh]
		scalar col7_convSE_`num'=_se[dhigh]
	
		di "Column 7, robust standard errors, awage<=`num'"
		reg dlnearn dhigh dLNandSE dpart dmarried down_exp if awage<=`num' & include==1, noconstant robust
		scalar col7_robSE_`num'=_se[dhigh]
	
		matrix define col7=(col7\col7_beta_`num'\col7_convSE_`num'\col7_robSE_`num')
	}
}

	* Column 8
matrix define col8=(0)
foreach num of numlist 999 90 75 65 60 {
	qui{

		di "Column 8, conventional standard errors, awage<=`num'"
		ivreg dlnearn (dhigh=dtwihi) dLNandSE dpart dmarried down_exp if awage<=`num' & include==1, noconstant
		scalar col8_beta_`num'=_b[dhigh]
		scalar col8_convSE_`num'=_se[dhigh]
	
		di "Column 8, robust standard errors, awage<=`num'"
		ivreg dlnearn (dhigh=dtwihi) dLNandSE dpart dmarried down_exp if awage<=`num' & include==1, noconstant robust
		scalar col8_robSE_`num'=_se[dhigh]
	
		matrix define col8=(col8\col8_beta_`num'\col8_convSE_`num'\col8_robSE_`num')
	}
}
***
* 3) Create a matrix containing all return to education estimates and corresponding standard errors in Table 1 and display the matrix
***
matrix define restr=(0\999\999\999\90\90\90\75\75\75\65\65\65\60\60\60)
matrix define results=(restr, col3, col4, col7, col8)
scalar rows=rowsof(results)
scalar cols=colsof(results)
matrix define results=(results[2..rows,1..cols])

matrix colnames results= "Restriction" "Column 3" "Column 4" "Column 7" "Column 8"
matrix rownames results= "beta" "convSE" "robustSE" "beta" "convSE" "robustSE" "beta" "convSE" "robustSE" "beta" "convSE" "robustSE" "beta" "convSE" "robustSE"

matrix list results

log close
